#pragma once

#include <ostream>
#include <mutex>

struct SafeOstream : std::ostream {
    SafeOstream( std::ostream &out, std::mutex &mutex ) :
        std::ostream( out.rdbuf() ),
        _mutex( mutex )
    {
        _mutex.lock();
    }
    ~SafeOstream() {
        _mutex.unlock();
    }
private:
    std::mutex &_mutex;
};

struct GuardedOstream {
    GuardedOstream( std::ostream &out ) :
        _out( out )
    {}

    SafeOstream stream() {
        return{ _out, _mutex };
    }

private:
    std::ostream &_out;
    std::mutex _mutex;
};

extern GuardedOstream gcout;
